@@ -2,7 +2,7 @@ context("xbart")
 
 source(system.file("common", "friedmanData.R", package = "dbarts"))
 
-test_that("xbart runs correctly with valid inputs", {
+test_that("random subsample runs correctly with valid inputs", {
   x <- testData$x
   y <- testData$y
   
@@ -12,7 +12,7 @@ test_that("xbart runs correctly with valid inputs", {
   power   <- c(1.5, 2)
   base    <- c(0.75, 0.8, 0.95)
 
-  xval <- xbart(x, y, n.samples = 15L, n.burn = c(10L, 3L, 1L),
+  xval <- xbart(x, y, n.samples = 15L, n.burn = c(10L, 3L, 1L), method = "random subsample",
                 n.reps = n.reps,
                 n.trees = n.trees,
                 k = k,
@@ -31,20 +31,75 @@ test_that("xbart runs correctly with valid inputs", {
     base    = as.character(base)))
 })
 
-test_that("xbart fails with invalid inputs", {
+test_that("k-fold runs correctly with valid inputs", {
   x <- testData$x
   y <- testData$y
   
+  n.reps  <- 8L
+  n.trees <- c(5L, 7L)
+  k       <- c(1, 2, 4)
+  power   <- c(1.5, 2)
+  base    <- c(0.75, 0.8, 0.95)
+
+  xval <- xbart(x, y, n.samples = 15L, n.burn = c(10L, 3L, 1L), method = "k-fold", n.test = 6,
+                n.reps = n.reps,
+                n.trees = n.trees,
+                k = k,
+                power = power,
+                base = base,
+                n.threads = 2L)
+  
+  expect_is(xval, "array")
+  expect_equal(dim(xval), c(n.reps, length(n.trees), length(k), length(power), length(base)))
+  expect_true(all(!is.na(xval)))
+  expect_equal(dimnames(xval), list(
+    rep     = NULL,
+    n.trees = as.character(n.trees),
+    k       = as.character(k),
+    power   = as.character(power),
+    base    = as.character(base)))
+})
+
+test_that("k-fold and random subsample are roughly similar", {
+  x <- testData$x
+  y <- testData$y
+  
+  k <- c(0.5, 8)
+  
+  set.seed(0)
+  xval.kf <- xbart(x, y, method = "k-fold",
+                   n.reps = 20L, n.samples = 100L, n.burn = c(100L, 50L, 10L),
+                   k = k,
+                   n.threads = 1L)
+
+  xval.rs <- xbart(x, y, method = "random subsample", 
+                   n.reps = 100L, n.samples = 100L, n.burn = c(100L, 50L, 10L),
+                   k = k, 
+                   n.threads = 1L)
+  
+  expect_true(sign(diff(apply(xval.rs, 2, mean))) == sign(diff(apply(xval.kf, 2, mean))))
+})
+
+test_that("fails with invalid inputs", {
+  x <- testData$x
+  y <- testData$y
+  
+  expect_error(xbart(y ~ x, method = "not-a-method"))
+  expect_error(xbart(y ~ x, method = NULL))
+  expect_error(xbart(y ~ x, method = NA_character_))
+  
   expect_error(xbart(y ~ x, n.samples = 0L))
   expect_error(xbart(y ~ x, n.samples = "not-a-integer"))
   expect_error(xbart(y ~ x, n.samples = NULL))
   expect_error(xbart(y ~ x, n.samples = NA_integer_))
   
-  expect_error(xbart(y ~ x, p.test = 0))
-  expect_error(xbart(y ~ x, p.test = length(testData$y) + 1))
-  expect_error(xbart(y ~ x, p.test = "not-a-numeric"))
-  expect_error(xbart(y ~ x, p.test = NULL))
-  expect_error(xbart(y ~ x, p.test = NA_real_))
+  expect_error(xbart(y ~ x, method = "k-fold", n.test = 1))
+  expect_error(xbart(y ~ x, method = "k-fold", n.test = length(testData$y) + 1))
+  expect_error(xbart(y ~ x, method = "random subsample", n.test = 0))
+  expect_error(xbart(y ~ x, method = "random subsample", n.test = length(testData$y) + 1))
+  expect_error(xbart(y ~ x, n.test = "not-a-numeric"))
+  expect_error(xbart(y ~ x, n.test = NULL))
+  expect_error(xbart(y ~ x, n.test = NA_real_))
   
   expect_error(xbart(y ~ x, n.reps = 0L))
   expect_error(xbart(y ~ x, n.reps = "not-a-integer"))
